#ifndef CA_2D_H
#define CA_2D_H

/// A single cellular automaton cell in a 2D array of cells.
class CA2D
{
public:
	CA2D(float xPosTopLeft, float yPosTopLeft, float xSize, float ySize, 
		float cornerRadius, float visualUpdateStepSize, float colorChangeTC, 
		float stateUpdateStepSize);

	~CA2D();

	void setNeighbor(size_t neighborIndex, CA2D* neighbor);

	bool isActive()const;

	void setActivity(bool active);

	void updateStatePhase1();

	void updateStatePhase2();

	void updateColor();

	void draw();

private:
	float mXPosTopLeft;
	float mYPosTopLeft;
	float mXSize;
	float mYSize;
	float mRadius;

	/// A scalar used to smoothly update the color.
	float mColorChangeConstant;

	/// The state update period.
	float mStateUpdateStepSize;

	/// A 2D cellular automaton has exactly 8 neighbors.
	CA2D* mNeighbors[8];

	/// The main binary "state."
	bool mActive;

	/// A separate stored copy of the next state used to enforce synchronous 
	/// updating.
	bool mNextStepActive;

	/// Cells stay active for a certain amount of time after being set active.
	float mActiveTimer;

	/// The cell's color which gradually tracks the binary state.
	float mColor;

	size_t mDisplayList;

private:
};

#endif
