#include "InputEventListener.h"

InputEventListener::InputEventListener(std::string host, int port1, int port2, 
	size_t screenSizeX, size_t screenSizeY)
	: DeviceEventListener("Input Event Listener")
{
	mDeviceSubscriber = new DeviceEventSubscriber(host, port1, port2);

	// Add all the device events we want to receive.
	mDeviceSubscriber->registerEvent(new TouchRawEvent(*(new std::vector<point>())));

	// Register this listener object with the device subscriber.
	mDeviceSubscriber->registerListener(this);

	pthread_mutex_init(&mQueueMutex, NULL);

	mScreenSizeXInverse = 1 / (float)screenSizeX;
	mScreenSizeYInverse = 1 / (float)screenSizeY;
}

InputEventListener::~InputEventListener()
{
}

void InputEventListener::process(DeviceEvent* deviceEvent)
{
	TouchRawEvent* touchEvent = dynamic_cast<TouchRawEvent*>(deviceEvent);
	if(touchEvent)
	{
		pthread_mutex_lock(&mQueueMutex);

		//std::cout << "TouchRawEvent:\n";
		size_t numPoints = touchEvent->getPoints().size();
		for(unsigned int i=0; i < numPoints; ++i)
		{
			point pt = touchEvent->getPoints().at(i);
			float normalizedX = pt.x * mScreenSizeXInverse;
			float normalizedY = pt.y * mScreenSizeYInverse;

			if (normalizedX < 0 || normalizedX > 1)
			{
				std::cout << "Ignoring invalid X pos (" << normalizedX << ")\n";
				continue;
			}

			if (normalizedY < 0 || normalizedY > 1)
			{
				std::cout << "Ignoring invalid Y pos (" << normalizedY << ")\n";
				continue;
			}

			mTouchPointQueue.push_front(TouchPoint(normalizedX, normalizedY));

/*
			//std::cout << "TouchRawEvent : (";
			point pt = touchEvent->getPoints().at(i);
			std::cout << "(" << pt.x << ", " << pt.y << ")\t";
			std::cout << "id(" << pt.id << ")\t";
			switch(pt.type)
			{
				case POINT_BIRTH:
					std::cout << "type(birth)\t";
					break;
				case POINT_DEATH:
					std::cout << "type(death)\t";
					break;
				case POINT_MOVE:
					std::cout << "type(move)\t";
					break;
				default:
					break;
			}
			std::cout << "side(" << pt.side << ")\n";
*/
		}
		//std::cout << "\n";

		pthread_mutex_unlock(&mQueueMutex);
	}
}

bool InputEventListener::hasUnhandledTouchPoints()
{
	pthread_mutex_lock(&mQueueMutex);

	if (!mTouchPointQueue.empty())
	{
		pthread_mutex_unlock(&mQueueMutex);
		return true;
	}
	else
	{
		pthread_mutex_unlock(&mQueueMutex);
		return false;
	}
}

void InputEventListener::popTouchPoint(float& x, float& y)
{
	pthread_mutex_lock(&mQueueMutex);

	x = mTouchPointQueue.back().x;
	y = mTouchPointQueue.back().y;
	mTouchPointQueue.pop_back();

	pthread_mutex_unlock(&mQueueMutex);
}
