#ifndef INPUT_EVENT_LISTENER_H
#define INPUT_EVENT_LISTENER_H

#include <iostream>
#include <string>
#include <list>
#include <stdlib.h>

#include <libsocketcpp/tcpsocket.h>

#include <DeviceCore/DeviceEventSubscriber/DeviceEventSubscriber.h>
#include <DeviceCore/DeviceEvent/DeviceEventListener/DeviceEventListener.h>

#include <Touch_InputDevice/DeviceEvents/TouchRawEvent.h>

/// A listener for multi-touch table events.
class InputEventListener : public DeviceEventListener
{
public:
	InputEventListener(std::string host, int port1, int port2, 
		size_t screenSizeX, size_t screenSizeY);

	virtual ~InputEventListener();

	/// The main event processing function.  This is called when a DeviceEvent 
	/// is received.
	virtual void process(DeviceEvent* deviceEvent);

	/// Returns true if the touch point queue is not empty.
	bool hasUnhandledTouchPoints();

	/// Returns the oldest touch point as x and y coordinates, normalized to 
	/// [0, 1].
	void popTouchPoint(float& x, float& y);

private:

	struct TouchPoint
	{
		TouchPoint(float xValue, float yValue)
		{
			x = xValue;
			y = yValue;
		}

		float x;
		float y;
	};

	DeviceEventSubscriber* mDeviceSubscriber;

	std::list<TouchPoint> mTouchPointQueue;
	pthread_mutex_t mQueueMutex;

	float mScreenSizeXInverse;
	float mScreenSizeYInverse;
};

#endif
