#include <math.h>
#include <assert.h>

#include "Tools.h"

void normalizeVec2D(float& x, float& y)
{
	float lengthSquared = x * x + y * y;
	if (lengthSquared > 0)
	{
		float lengthInverse = 1 / ::sqrtf(lengthSquared);
		x *= lengthInverse;
		y *= lengthInverse;
	}
	else
	{
		std::cout << "Warning: Cannot normalize zero-length vector\n";
	}
}

void rotateVec2D(float& x, float& y, float angleDegrees)
{
	const float piOver180 = 0.017453292519943295f;
	float angleRadians = angleDegrees * piOver180;
	float cosAngle = cos(angleRadians);
	float sinAngle = sin(angleRadians);
	float newX = cosAngle * x - sinAngle * y;
	float newY = sinAngle * x + cosAngle * y;
	x = newX;
	y = newY;
}

float randomRealUniform(float min, float max)
{
	return (float)rand() / RAND_MAX * (max - min) + min;
}

float randomGauss(float variance)
{
	/// Adapted from Numerical Recipes in C, Second Edition.

	static int iset = 0;
	static float gset;
	float fac, rsq, v1, v2;

	if (0 == iset)
	{
		do
		{
			static const float invDenominator = 
				1.0f / (RAND_MAX + 1.0f);
			v1 = 2.0f * (rand() * invDenominator) - 1.0f;
			v2 = 2.0f * (rand() * invDenominator) - 1.0f;
			rsq = v1 * v1 + v2 * v2;
		}
		while (rsq >= 1.0 || rsq == 0.0);

		fac = ::sqrt(-2.0f * ::log(rsq) / rsq);
		gset = v1 * fac;
		iset = 1;
		return v2 * fac * variance;
	}
	else
	{
		iset = 0;
		return gset * variance;
	}
}
